package net.locusworks.lsproject.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import net.locusworks.lsproject.driver.Program;
import net.locusworks.lsproject.object.Network;
import net.locusworks.lsproject.object.Router;


public class CostViewerWindow extends JPanel implements TableModelListener{
	/**
	 * 
	 */
	private static final long serialVersionUID = 8571350255455457432L;
	private boolean DEBUG = false;
	private Router router = null;

	
	public CostViewerWindow(Router _router){
		super(new BorderLayout());
		
		router = _router;
		
		String[] columnNames = {"Connection",
				"Cost"};
		
		int numberOfConnections = router.getConnections().size();
		
		 Object data[][]= new Object[numberOfConnections][2];
		
		for (int i = 0; i < numberOfConnections; i++){
			
			String tempFirstParticipant = router.getConnections()
			.get(i).getFirstParticipantAddress();
			
			String tempSecondParticipant = router.getConnections()
			.get(i).getSecondParticipantAddress();
			
			if(tempFirstParticipant.equals(router.getIpAddress())){
				data[i][0] = tempSecondParticipant;
			}
			else{
				data[i][0] = tempFirstParticipant;
			}
			data[i][1] = router.getConnections().get(i).getCost();
		}
		
		final JTable table = new JTable(data, columnNames);
		table.setPreferredScrollableViewportSize(new Dimension(300, 70));
        table.setFillsViewportHeight(true);
		table.getModel().addTableModelListener(this);
		
		JScrollPane scrollPane = new JScrollPane(table);
		add(scrollPane);
		
	}
	
	public void createGUI(){
		
		JFrame frame = new JFrame("Cost Viewer For Router " + router.getIpAddress());		
		CostViewerWindow newContentPane = new CostViewerWindow(router);
		newContentPane.setOpaque(true);
		frame.setContentPane(newContentPane);
		frame.pack();
		frame.setLocationRelativeTo(null);
		frame.setVisible(true);
	}

	@Override
	public void tableChanged(TableModelEvent e) {
		if (e.getColumn() == 0) {
			return;
		}
		
	 	int row = e.getFirstRow();
        int column = e.getColumn();
        TableModel model = (TableModel)e.getSource();
        Object data = model.getValueAt(row, column);
        
        final String otherRouterIp = model.getValueAt(row, 0).toString();
      
        final int oldCost = router.getConnectionByParticipant(router.getIpAddress(), otherRouterIp).getCost();
        int newCost = oldCost;
        try {
        	newCost = Integer.parseInt(data.toString());
        	
        	if (newCost < 1 || newCost > 10){
            	System.out.println("Cost Has To Be Between 0 - 10");
            	
            	newCost = oldCost;
            	
            	return;
            }
            	
        	router.getConnectionByParticipant(
        			router.getIpAddress(), otherRouterIp).setCost(newCost);
        	Router otherRouter = Network.getInstance().getRouterByIp(otherRouterIp);
        	otherRouter.getConnectionByParticipant(
        			router.getIpAddress(), otherRouterIp).setCost(newCost);
        }
        catch (NumberFormatException ex) {
        	System.out.println("Invalid cost.");
        }
	}
		
}


