package net.locusworks.lsproject.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;

import net.locusworks.lsproject.driver.Program;
import net.locusworks.lsproject.gui.helper.ImageHelper;
import net.locusworks.lsproject.object.Network;
import net.locusworks.lsproject.util.Utility;


/**
 * The first window that the user will see.  Prompts user to create a new network, load a network, or exit.
 */
@SuppressWarnings("serial")
public class StartupWindow extends JFrame {
	// Declare class members.
	private JButton newNetworkButton;
	private JButton loadNetworkButton;
	private JButton exitButton;
	
	/**
	 * Construct a new startup window.
	 */
	public StartupWindow() {
		super("LS Network Project");
		
		initializeGui();
	}
	
	/**
	 * Set up the GUI for this window.
	 */
	private void initializeGui() {
		// Initialize class members (GUI objects)
		newNetworkButton 	= new JButton("New Network");
		loadNetworkButton 	= new JButton("Load Network");
		exitButton 			= new JButton("Exit");
		
		// Layout setup.
		setLayout(new GridBagLayout());
		
		// Component layout setup.
		GridBagConstraints c = new GridBagConstraints();
		
		c.gridx = 0;
		c.gridy = 0;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(10, 20, 5, 20);
		add(newNetworkButton, c);
		
		c.gridy = 1;
		c.insets = new Insets(5, 20, 5, 20);
		add(loadNetworkButton, c);
		
		c.gridy = 2;
		c.insets = new Insets(5, 20, 10, 20);
		add(exitButton, c);
		
		// Event handler setup.
		newNetworkButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				newNetworkButton_onClick();
			}
		});
		
		loadNetworkButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				loadNetworkButton_onClick();
			}
		});
		
		exitButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				exitButton_onClick();
			}
		});
		
		 // Load window icon.
    setIconImage(ImageHelper.getImageFromResource("assets/router_ico.png"));
				
		// Typical window setup.
		pack();
		
		setLocationRelativeTo(null);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setVisible(true);
	}
	
	/**
	 * Handle event when the newNetworkButton is clicked.
	 * Opens a new RouterViewerWindow GUI and closes this window.
	 */
	private void newNetworkButton_onClick() {
		setVisible(false);
		
		dispose();
		
		String networkName = "";
		while (networkName.equals("")) {
			networkName = JOptionPane.showInputDialog("Please enter your network's name:");
			if (networkName == null) {
				// Canceled.
				System.exit(0);
			}
		}
		
		Network.createInstance(networkName);
		
		new RouterViewerWindow();
	}
	
	/**
	 * Handle event when the loadNetworkButton is clicked.
	 * Opens a file dialog, loads a network, opens a new RouterViewerWindow GUI and closes this window.
	 */
	private void loadNetworkButton_onClick() {
		JFileChooser fileChooser = new JFileChooser(".");
		fileChooser.addChoosableFileFilter(new FileFilter() {
			@Override public boolean accept(File f) {
				if (f.isDirectory())
					return true;
				
				final String ext = Utility.getExtension(f);
				if (ext == null) {
					return false;
				}
				
				return !ext.equalsIgnoreCase(Utility.FILE_EXTENSION);
			}

			@Override public String getDescription() {
				return "*" + Utility.FILE_EXTENSION + " - Serialized network object file.";
			}
			
		});
		
		final int userAction = fileChooser.showOpenDialog(this);
		if (userAction == JFileChooser.CANCEL_OPTION) {
			// Canceled.
			return;
		}
		
		File f = fileChooser.getSelectedFile();
		
		try {
			Utility.loadNetwork(f);
		}
		catch (IOException e) {
			e.printStackTrace();
			
			JOptionPane.showMessageDialog(this, 
				"An error occured while trying to load the network file:\n" + e.getMessage(), 
				"Error loading file!", 
				JOptionPane.ERROR_MESSAGE);
			
			return;
		}
		
		setVisible(false);
		
		dispose();
		
		new RouterViewerWindow();
	}
	
	/**
	 * Handle event when exitButton is clicked.
	 * Closes the program.
	 */
	private void exitButton_onClick() {
		System.exit(0);
	}
}
