package net.locusworks.lsproject.gui.helper;

import net.locusworks.lsproject.object.DijkstraEngine;
import net.locusworks.lsproject.object.Path;
import net.locusworks.lsproject.object.Router;

/**
 * The RouterPath class's job is to keep track of two separate routers for the purpose of
 * showing the quickest path from the source (firstRouter) to the destination (secondRouter).
 */
public class RouterPath {
	public static Router firstRouter = null;
	public static Router secondRouter = null;
	
	public static Path highlightedPath = null;
	
	/**
	 * Null out the routers.
	 */
	public static void clear() {
		firstRouter = null;
		secondRouter = null;
		highlightedPath = null;
		
		System.out.println("Cleared highlights.");
	}
	
	/**
	 * Highlights a router. If one is highlighted, it highlights the other one.
	 * If both are highlighted, it nulls the second and highlights the first.
	 * @param r Router to highlight.
	 */
	public static void highlightOne(Router r) {
		if (firstRouter == null) {
			firstRouter = r;
			System.out.println("Highlighted source router: " + r.getIpAddress()); 
		}
		else if (secondRouter == null) {
			secondRouter = r;
			System.out.println("Highlighted destination router: " + r.getIpAddress());
			
			highlightedPath = findShortestPath(firstRouter, secondRouter);
			if (highlightedPath == null) {
				System.out.println("No connection from " + 
					firstRouter.getIpAddress() + " to " + secondRouter.getIpAddress());
				
				clear();
			}
			else {
				System.out.println("Path: total cost: " + highlightedPath.getTotalCost());
			}
		}
		else {
			clear();
		}
		
		if (firstRouter == secondRouter && firstRouter != null) {
			clear();
		}
	}
	
	/**
	 * Find the shortest path from the first router (A) to the target router (B).
	 * @see DijkstraEngine.findShortestPath
	 * @param _firstRouter Starting router.
	 * @param _secondRouter Target router.
	 * @return Sequence from the source router to the destination router.
	 */
	public static Path findShortestPath(Router _firstRouter, Router _secondRouter) {
		return new DijkstraEngine().findShortestPath(_firstRouter, _secondRouter);
	}
}
